function Prob = DIR_COMBINE(Dir_para,L_combine,Eval_point)

% Purpose: 
% Compute the c.d.f. of linear combination of Dirichlet variates
% -----------------------------------
% Algorithm:
% "Bayesian Inference with Monotone Instrumental Variables"
% by Hang Qian (2011)
% -----------------------------------
% Usage:
% Dir_para= parameters of the Dirichlet distribution, a column vector
% L_combine = Linear combination coefficients of Dirichlet variates, a column vector
% Eval_point = points of evaluation for the c.d.f.
% -----------------------------------
% Returns:
% Prob = c.d.f. of linear combination of Dirichlet variates
% 
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com

Dir_para = Dir_para(:);
L_combine = L_combine(:);

% Truncate for (0,Inf)
power = 15;
grid = exp(-power:power);
test1 = abs(INTEGRAND(grid,Dir_para,L_combine,Eval_point)) < 1e-8;
if all(test1 == 1)
    truncate = exp(-power);
else
    test2 = find(diff(test1) == 1);
    if isempty(test2)
        truncate = exp(power);
    else
        truncate = grid(test2(end)+1);
    end
end
   
INT = GAUSS_LEGENDRE(@(t) INTEGRAND(t,Dir_para,L_combine,Eval_point),0,truncate,90);
Prob = 0.5 + INT;

end


%-------------------------
%  INTEGRAND
%-------------------------
function Int = INTEGRAND(t,Dir_para,L_combine,Eval_point)
Char_fn = 1 - (1i) * (L_combine - Eval_point) * t;
radius = abs(Char_fn);
theta = angle(Char_fn);
int_part1 = exp(-Dir_para' * log(radius));
int_part2 = sin(Dir_para' * theta);
Int = 1./(pi * t) .* int_part1 .* int_part2;
end