function HPD_region = HPD_CDF(x,cdf,alpha)

% Purpose: 
% Compute the HPD region using the c.d.f. of a distribution
% -----------------------------------
% Algorithm: 
% Grid search, pick the highest density
% -----------------------------------
% Usage:
% x = points of evaluation
% cdf = c.d.f. evaluated at x
% alpha = significance level, say 0.05, (In that case, 95% HPD interval will be computed)

% -----------------------------------
% Returns:
% HPD_region = an n-by-2 matrix, where n is the number of intervals. 
%   Single modal distribution yields a 1-by-2 vector in the form of [lb,ub]
% 
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


base = x(2) - x(1);
prob = [diff(cdf);0];
[prob_sort,ind] = sort(prob,'descend');
prob_sum = cumsum(prob_sort);
if prob_sum(end) < 0.99 || prob_sum(end) > 1.01
    disp('The c.d.f. you provided may not be proper.')
    disp(['Normalizing constant = ',num2str(prob_sum(end)),' .Try to make the distribution proper.'])
    prob_sum = prob_sum / prob_sum(end);
end
HPD_ind = ind(prob_sum <= (1-alpha));
HPD_grid = x(HPD_ind);
HPD_grid = sort(HPD_grid);

check_modal = find( diff(HPD_grid) > base * 1.001);
nmodal = length(check_modal) + 1;
if nmodal == 1
    HPD_region = [HPD_grid(1),HPD_grid(end)];
else
    HPD_region = zeros(nmodal,2);
    HPD_region(1,:) = [HPD_grid(1), HPD_grid(check_modal(1))];    
    for m = 2:nmodal-1
        HPD_region(m,:) = [HPD_grid(check_modal(m-1)+1), HPD_grid(check_modal(m))];
    end
    HPD_region(nmodal,:) = [HPD_grid(check_modal(end)+1), HPD_grid(end)];
end

[largest,ind] = max(HPD_region(:,2) - HPD_region(:,1));
HPD_region = HPD_region(ind,:);


